/**
 * @file:   SedAlgorithmParameter.h
 * @brief:  Implementation of the SedAlgorithmParameter class
 * @author: Frank T. Bergmann
 *
 * <!--------------------------------------------------------------------------
 * This file is part of libSEDML.  Please visit http://sed-ml.org for more
 * information about SED-ML. The latest version of libSEDML can be found on
 * github: https://github.com/fbergmann/libSEDML/
 *
 * Copyright (c) 2013-2016, Frank T. Bergmann
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ------------------------------------------------------------------------ -->
 */


#ifndef SedAlgorithmParameter_H__
#define SedAlgorithmParameter_H__


#include <sedml/common/extern.h>
#include <sedml/common/sedmlfwd.h>


#ifdef __cplusplus


#include <string>


#include <sedml/SedBase.h>
#include <sedml/SedListOf.h>
#include <sedml/SedNamespaces.h>




LIBSEDML_CPP_NAMESPACE_BEGIN


class LIBSEDML_EXTERN SedAlgorithmParameter : public SedBase
{

protected:

  std::string   mKisaoID;
  std::string   mValue;


public:

  /**
   * Creates a new SedAlgorithmParameter with the given level, version, and package version.
   *
   * @param level an unsigned int, the SEDML Level to assign to this SedAlgorithmParameter
   *
   * @param version an unsigned int, the SEDML Version to assign to this SedAlgorithmParameter
   *
   * @param pkgVersion an unsigned int, the SEDML Sed Version to assign to this SedAlgorithmParameter
   */
  SedAlgorithmParameter(unsigned int level      = SEDML_DEFAULT_LEVEL,
                        unsigned int version    = SEDML_DEFAULT_VERSION);


  /**
   * Creates a new SedAlgorithmParameter with the given SedNamespaces object.
   *
   * @param sedns the SedNamespaces object
   */
  SedAlgorithmParameter(SedNamespaces* sedns);


  /**
   * Copy constructor for SedAlgorithmParameter.
   *
   * @param orig; the SedAlgorithmParameter instance to copy.
   */
  SedAlgorithmParameter(const SedAlgorithmParameter& orig);


  /**
   * Assignment operator for SedAlgorithmParameter.
   *
   * @param rhs; the object whose values are used as the basis
   * of the assignment
   */
  SedAlgorithmParameter& operator=(const SedAlgorithmParameter& rhs);


  /**
   * Creates and returns a deep copy of this SedAlgorithmParameter object.
   *
   * @return a (deep) copy of this SedAlgorithmParameter object.
   */
  virtual SedAlgorithmParameter* clone() const;


  /**
   * Destructor for SedAlgorithmParameter.
   */
  virtual ~SedAlgorithmParameter();


  /**
   * Returns the value of the "kisaoID" attribute of this SedAlgorithmParameter.
   *
   * @return the value of the "kisaoID" attribute of this SedAlgorithmParameter as a string.
   */
  virtual const std::string& getKisaoID() const;


  /**
   * Predicate returning @c true or @c false depending on whether this
   * SedAlgorithmParameter's "kisaoID" attribute has been set.
   *
   * @return @c true if this SedAlgorithmParameter's "kisaoID" attribute has been set,
   * otherwise @c false is returned.
   */
  virtual bool isSetKisaoID() const;


  /**
   * Sets the value of the "kisaoID" attribute of this SedAlgorithmParameter.
   *
   * @param kisaoID; const std::string& value of the "kisaoID" attribute to be set
   *
   * @return integer value indicating success/failure of the
   * function.  @if clike The value is drawn from the
   * enumeration #OperationReturnValues_t. @endif The possible values
   * returned by this function are:
   * @li LIBSEDML_OPERATION_SUCCESS
   * @li LIBSEDML_INVALID_ATTRIBUTE_VALUE
   */
  virtual int setKisaoID(const std::string& kisaoID);


  /**
   * Unsets the value of the "kisaoID" attribute of this SedAlgorithmParameter.
   *
   * @return integer value indicating success/failure of the
   * function.  @if clike The value is drawn from the
   * enumeration #OperationReturnValues_t. @endif The possible values
   * returned by this function are:
   * @li LIBSEDML_OPERATION_SUCCESS
   * @li LIBSEDML_OPERATION_FAILED
   */
  virtual int unsetKisaoID();


  /**
   * Returns the value of the "value" attribute of this SedAlgorithmParameter.
   *
   * @return the value of the "value" attribute of this SedAlgorithmParameter as a string.
   */
  virtual const std::string& getValue() const;


  /**
   * Predicate returning @c true or @c false depending on whether this
   * SedAlgorithmParameter's "value" attribute has been set.
   *
   * @return @c true if this SedAlgorithmParameter's "value" attribute has been set,
   * otherwise @c false is returned.
   */
  virtual bool isSetValue() const;


  /**
   * Sets the value of the "value" attribute of this SedAlgorithmParameter.
   *
   * @param value; const std::string& value of the "value" attribute to be set
   *
   * @return integer value indicating success/failure of the
   * function.  @if clike The value is drawn from the
   * enumeration #OperationReturnValues_t. @endif The possible values
   * returned by this function are:
   * @li LIBSEDML_OPERATION_SUCCESS
   * @li LIBSEDML_INVALID_ATTRIBUTE_VALUE
   */
  virtual int setValue(const std::string& value);


  /**
   * Unsets the value of the "value" attribute of this SedAlgorithmParameter.
   *
   * @return integer value indicating success/failure of the
   * function.  @if clike The value is drawn from the
   * enumeration #OperationReturnValues_t. @endif The possible values
   * returned by this function are:
   * @li LIBSEDML_OPERATION_SUCCESS
   * @li LIBSEDML_OPERATION_FAILED
   */
  virtual int unsetValue();


  /**
   * Returns the XML element name of this object, which for SedAlgorithmParameter, is
   * always @c "sedAlgorithmParameter".
   *
   * @return the name of this element, i.e. @c "sedAlgorithmParameter".
   */
  virtual const std::string& getElementName() const;


  /**
   * Returns the libSEDML type code for this SEDML object.
   *
   * @if clike LibSEDML attaches an identifying code to every kind of SEDML
   * object.  These are known as <em>SEDML type codes</em>.  The set of
   * possible type codes is defined in the enumeration #SEDMLTypeCode_t.
   * The names of the type codes all begin with the characters @c
   * SEDML_. @endif@if java LibSEDML attaches an identifying code to every
   * kind of SEDML object.  These are known as <em>SEDML type codes</em>.  In
   * other languages, the set of type codes is stored in an enumeration; in
   * the Java language interface for libSEDML, the type codes are defined as
   * static integer constants in the interface class {@link
   * libsedmlConstants}.  The names of the type codes all begin with the
   * characters @c SEDML_. @endif@if python LibSEDML attaches an identifying
   * code to every kind of SEDML object.  These are known as <em>SEDML type
   * codes</em>.  In the Python language interface for libSEDML, the type
   * codes are defined as static integer constants in the interface class
   * @link libsedml@endlink.  The names of the type codes all begin with the
   * characters @c SEDML_. @endif@if csharp LibSEDML attaches an identifying
   * code to every kind of SEDML object.  These are known as <em>SEDML type
   * codes</em>.  In the C# language interface for libSEDML, the type codes
   * are defined as static integer constants in the interface class @link
   * libsedmlcs.libsedml@endlink.  The names of the type codes all begin with
   * the characters @c SEDML_. @endif
   *
   * @return the SEDML type code for this object, or
   * @link SEDMLTypeCode_t#SEDML_UNKNOWN SEDML_UNKNOWN@endlink (default).
   *
   * @see getElementName()
   */
  virtual int getTypeCode() const;


  /**
   * Predicate returning @c true if all the required attributes
   * for this SedAlgorithmParameter object have been set.
   *
   * @note The required attributes for a SedAlgorithmParameter object are:
   * @li "kisaoID"
   * @li "value"
   *
   * @return a boolean value indicating whether all the required
   * attributes for this object have been defined.
   */
  virtual bool hasRequiredAttributes() const;


  /** @cond doxygen-libsedml-internal */

  /**
   * Subclasses should override this method to write out their contained
   * SEDML objects as XML elements.  Be sure to call your parents
   * implementation of this method as well.
   */
  virtual void writeElements(XMLOutputStream& stream) const;


  /** @endcond doxygen-libsedml-internal */


  /** @cond doxygen-libsedml-internal */

  /**
   * Accepts the given SedVisitor.
   */
  virtual bool accept(SedVisitor& v) const;


  /** @endcond doxygen-libsedml-internal */


  /** @cond doxygen-libsedml-internal */

  /**
   * Sets the parent SedDocument.
   */
  virtual void setSedDocument(SedDocument* d);


  /** @endcond doxygen-libsedml-internal */


protected:

  /** @cond doxygen-libsedml-internal */

  /**
   * Get the list of expected attributes for this element.
   */
  virtual void addExpectedAttributes(ExpectedAttributes& attributes);


  /** @endcond doxygen-libsedml-internal */


  /** @cond doxygen-libsedml-internal */

  /**
   * Read values from the given XMLAttributes set into their specific fields.
   */
  virtual void readAttributes(const XMLAttributes& attributes,
                              const ExpectedAttributes& expectedAttributes);


  /** @endcond doxygen-libsedml-internal */


  /** @cond doxygen-libsedml-internal */

  /**
   * Write values of XMLAttributes to the output stream.
   */
  virtual void writeAttributes(XMLOutputStream& stream) const;


  /** @endcond doxygen-libsedml-internal */


public:

  /**
   * @returns the KisaoTerm as integer if possible or -1 in
   * case of failure
   */
  int getKisaoIDasInt() const;

  /**
   * Sets the KisaoId as integer
   */
  int setKisaoID(int kisaoID);
};

class LIBSEDML_EXTERN SedListOfAlgorithmParameters : public SedListOf
{

public:

  /**
   * Creates a new SedListOfAlgorithmParameters with the given level, version, and package version.
   *
   * @param level an unsigned int, the SEDML Level to assign to this SedListOfAlgorithmParameters
   *
   * @param version an unsigned int, the SEDML Version to assign to this SedListOfAlgorithmParameters
   *
   * @param pkgVersion an unsigned int, the SEDML Sed Version to assign to this SedListOfAlgorithmParameters
   */
  SedListOfAlgorithmParameters(unsigned int level      = SEDML_DEFAULT_LEVEL,
                               unsigned int version    = SEDML_DEFAULT_VERSION
                              );


  /**
   * Creates a new SedListOfAlgorithmParameters with the given SedNamespaces object.
   *
   * @param sedns the SedNamespaces object
   */
  SedListOfAlgorithmParameters(SedNamespaces* sedns);


  /**
   * Creates and returns a deep copy of this SedListOfAlgorithmParameters object.
   *
   * @return a (deep) copy of this SedListOfAlgorithmParameters object.
   */
  virtual SedListOfAlgorithmParameters* clone() const;


  /**
   * Get a AlgorithmParameter from the SedListOfAlgorithmParameters.
   *
   * @param n the index number of the AlgorithmParameter to get.
   *
   * @return the nth AlgorithmParameter in this SedListOfAlgorithmParameters.
   *
   * @see size()
   */
  virtual SedAlgorithmParameter* get(unsigned int n);


  /**
   * Get a AlgorithmParameter from the SedListOfAlgorithmParameters.
   *
   * @param n the index number of the AlgorithmParameter to get.
   *
   * @return the nth AlgorithmParameter in this SedListOfAlgorithmParameters.
   *
   * @see size()
   */
  virtual const SedAlgorithmParameter* get(unsigned int n) const;


  /**
   * Get a AlgorithmParameter from the SedListOfAlgorithmParameters
   * based on its identifier.
   *
   * @param sid a string representing the identifier
   * of the AlgorithmParameter to get.
   *
   * @return AlgorithmParameter in this SedListOfAlgorithmParameters
   * with the given id or NULL if no such
   * AlgorithmParameter exists.
   *
   * @see get(unsigned int n)   *
   * @see size()
   */
  virtual SedAlgorithmParameter* get(const std::string& sid);


  /**
   * Get a AlgorithmParameter from the SedListOfAlgorithmParameters
   * based on its identifier.
   *
   * @param sid a string representing the identifier
   * of the AlgorithmParameter to get.
   *
   * @return AlgorithmParameter in this SedListOfAlgorithmParameters
   * with the given id or NULL if no such
   * AlgorithmParameter exists.
   *
   * @see get(unsigned int n)   *
   * @see size()
   */
  virtual const SedAlgorithmParameter* get(const std::string& sid) const;


  /**
   * Adds a copy the given "AlgorithmParameter" to this SedListOfAlgorithmParameters.
   *
   * @param ap; the AlgorithmParameter object to add
   *
   * @return integer value indicating success/failure of the
   * function.  @if clike The value is drawn from the
   * enumeration #OperationReturnValues_t. @endif The possible values
   * returned by this function are:
   * @li LIBSEDML_OPERATION_SUCCESS
   * @li LIBSEDML_INVALID_ATTRIBUTE_VALUE
   */
  int addAlgorithmParameter(const SedAlgorithmParameter* ap);


  /**
   * Get the number of AlgorithmParameter objects in this SedListOfAlgorithmParameters.
   *
   * @return the number of AlgorithmParameter objects in this SedListOfAlgorithmParameters
   */
  unsigned int getNumAlgorithmParameters() const;


  /**
   * Creates a new AlgorithmParameter object, adds it to the
   * SedListOfAlgorithmParameters and returns the AlgorithmParameter object created.
   *
   * @return a new AlgorithmParameter object instance
   *
   * @see addAlgorithmParameter(const SedAlgorithmParameter* ap)
   */
  SedAlgorithmParameter* createAlgorithmParameter();


  /**
   * Removes the nth AlgorithmParameter from this SedListOfAlgorithmParameters
   * and returns a pointer to it.
   *
   * The caller owns the returned item and is responsible for deleting it.
   *
   * @param n the index of the AlgorithmParameter to remove.
   *
   * @see size()
   */
  virtual SedAlgorithmParameter* remove(unsigned int n);


  /**
   * Removes the AlgorithmParameter from this SedListOfAlgorithmParameters with the given identifier
   * and returns a pointer to it.
   *
   * The caller owns the returned item and is responsible for deleting it.
   * If none of the items in this list have the identifier @p sid, then
   * @c NULL is returned.
   *
   * @param sid the identifier of the AlgorithmParameter to remove.
   *
   * @return the AlgorithmParameter removed. As mentioned above, the caller owns the
   * returned item.
   */
  virtual SedAlgorithmParameter* remove(const std::string& sid);


  /**
   * Returns the XML element name of this object, which for SedListOfAlgorithmParameters, is
   * always @c "sedListOfAlgorithmParameters".
   *
   * @return the name of this element, i.e. @c "sedListOfAlgorithmParameters".
   */
  virtual const std::string& getElementName() const;


  /**
   * Returns the libSEDML type code for this SEDML object.
   *
   * @if clike LibSEDML attaches an identifying code to every kind of SEDML
   * object.  These are known as <em>SEDML type codes</em>.  The set of
   * possible type codes is defined in the enumeration #SEDMLTypeCode_t.
   * The names of the type codes all begin with the characters @c
   * SEDML_. @endif@if java LibSEDML attaches an identifying code to every
   * kind of SEDML object.  These are known as <em>SEDML type codes</em>.  In
   * other languages, the set of type codes is stored in an enumeration; in
   * the Java language interface for libSEDML, the type codes are defined as
   * static integer constants in the interface class {@link
   * libsedmlConstants}.  The names of the type codes all begin with the
   * characters @c SEDML_. @endif@if python LibSEDML attaches an identifying
   * code to every kind of SEDML object.  These are known as <em>SEDML type
   * codes</em>.  In the Python language interface for libSEDML, the type
   * codes are defined as static integer constants in the interface class
   * @link libsedml@endlink.  The names of the type codes all begin with the
   * characters @c SEDML_. @endif@if csharp LibSEDML attaches an identifying
   * code to every kind of SEDML object.  These are known as <em>SEDML type
   * codes</em>.  In the C# language interface for libSEDML, the type codes
   * are defined as static integer constants in the interface class @link
   * libsedmlcs.libsedml@endlink.  The names of the type codes all begin with
   * the characters @c SEDML_. @endif
   *
   * @return the SEDML type code for this object, or
   * @link SEDMLTypeCode_t#SEDML_UNKNOWN SEDML_UNKNOWN@endlink (default).
   *
   * @see getElementName()
   */
  virtual int getTypeCode() const;


  /**
   * Returns the libSEDML type code for the SEDML objects
   * contained in this SedListOf object
   *
   * @if clike LibSEDML attaches an identifying code to every kind of SEDML
   * object.  These are known as <em>SEDML type codes</em>.  The set of
   * possible type codes is defined in the enumeration #SEDMLTypeCode_t.
   * The names of the type codes all begin with the characters @c
   * SEDML_. @endif@if java LibSEDML attaches an identifying code to every
   * kind of SEDML object.  These are known as <em>SEDML type codes</em>.  In
   * other languages, the set of type codes is stored in an enumeration; in
   * the Java language interface for libSEDML, the type codes are defined as
   * static integer constants in the interface class {@link
   * libsedmlConstants}.  The names of the type codes all begin with the
   * characters @c SEDML_. @endif@if python LibSEDML attaches an identifying
   * code to every kind of SEDML object.  These are known as <em>SEDML type
   * codes</em>.  In the Python language interface for libSEDML, the type
   * codes are defined as static integer constants in the interface class
   * @link libsedml@endlink.  The names of the type codes all begin with the
   * characters @c SEDML_. @endif@if csharp LibSEDML attaches an identifying
   * code to every kind of SEDML object.  These are known as <em>SEDML type
   * codes</em>.  In the C# language interface for libSEDML, the type codes
   * are defined as static integer constants in the interface class @link
   * libsedmlcs.libsedml@endlink.  The names of the type codes all begin with
   * the characters @c SEDML_. @endif
   *
   * @return the SEDML type code for the objects in this SedListOf instance, or
   * @link SEDMLTypeCode_t#SEDML_UNKNOWN SEDML_UNKNOWN@endlink (default).
   *
   * @see getElementName()
   */
  virtual int getItemTypeCode() const;


protected:

  /** @cond doxygen-libsedml-internal */

  /**
   * Creates a new AlgorithmParameter in this SedListOfAlgorithmParameters
   */
  virtual SedBase* createObject(XMLInputStream& stream);


  /** @endcond doxygen-libsedml-internal */


  /** @cond doxygen-libsedml-internal */

  /**
   * Write the namespace for the Sed package.
   */
  virtual void writeXMLNS(XMLOutputStream& stream) const;


  /** @endcond doxygen-libsedml-internal */



};



LIBSEDML_CPP_NAMESPACE_END

#endif  /*  __cplusplus  */

#ifndef SWIG

LIBSEDML_CPP_NAMESPACE_BEGIN
BEGIN_C_DECLS

LIBSEDML_EXTERN
SedAlgorithmParameter_t *
SedAlgorithmParameter_create(unsigned int level, unsigned int version);


LIBSEDML_EXTERN
void
SedAlgorithmParameter_free(SedAlgorithmParameter_t * sap);


LIBSEDML_EXTERN
SedAlgorithmParameter_t *
SedAlgorithmParameter_clone(SedAlgorithmParameter_t * sap);


LIBSEDML_EXTERN
char *
SedAlgorithmParameter_getKisaoID(SedAlgorithmParameter_t * sap);


LIBSEDML_EXTERN
char *
SedAlgorithmParameter_getValue(SedAlgorithmParameter_t * sap);


LIBSEDML_EXTERN
int
SedAlgorithmParameter_isSetKisaoID(SedAlgorithmParameter_t * sap);


LIBSEDML_EXTERN
int
SedAlgorithmParameter_isSetValue(SedAlgorithmParameter_t * sap);


LIBSEDML_EXTERN
int
SedAlgorithmParameter_setKisaoID(SedAlgorithmParameter_t * sap, const char * kisaoID);


LIBSEDML_EXTERN
int
SedAlgorithmParameter_setValue(SedAlgorithmParameter_t * sap, const char * value);


LIBSEDML_EXTERN
int
SedAlgorithmParameter_unsetKisaoID(SedAlgorithmParameter_t * sap);


LIBSEDML_EXTERN
int
SedAlgorithmParameter_unsetValue(SedAlgorithmParameter_t * sap);


LIBSEDML_EXTERN
int
SedAlgorithmParameter_hasRequiredAttributes(SedAlgorithmParameter_t * sap);


LIBSEDML_EXTERN
SedAlgorithmParameter_t *
SedListOfAlgorithmParameters_getById(SedListOf_t * lo, const char * sid);


LIBSEDML_EXTERN
SedAlgorithmParameter_t *
SedListOfAlgorithmParameters_removeById(SedListOf_t * lo, const char * sid);




END_C_DECLS
LIBSEDML_CPP_NAMESPACE_END

#endif  /*  !SWIG  */

#endif /*  SedAlgorithmParameter_H__  */

